@@ -19,7 +19,6 @@ use {
         clock::{Clock, Slot},
         entrypoint::{ProgramResult, SUCCESS},
         epoch_schedule::EpochSchedule,
-        feature_set::demote_sysvar_write_locks,
         fee_calculator::{FeeCalculator, FeeRateGovernor},
         genesis_config::{ClusterType, GenesisConfig},
         hash::Hash,
@@ -259,14 +258,12 @@ impl solana_sdk::program_stubs::SyscallStubs for SyscallStubs {
             }
             panic!("Program id {} wasn't found in account_infos", program_id);
         };
-        let demote_sysvar_write_locks =
-            invoke_context.is_feature_active(&demote_sysvar_write_locks::id());
         // TODO don't have the caller's keyed_accounts so can't validate writer or signer escalation or deescalation yet
         let caller_privileges = message
             .account_keys
             .iter()
             .enumerate()
-            .map(|(i, _)| message.is_writable(i, demote_sysvar_write_locks))
+            .map(|(i, _)| message.is_writable(i))
             .collect::<Vec<bool>>();
 
         stable_log::program_invoke(&logger, &program_id, invoke_context.invoke_depth());
@@ -290,7 +287,7 @@ impl solana_sdk::program_stubs::SyscallStubs for SyscallStubs {
         'outer: for key in &message.account_keys {
             for account_info in account_infos {
                 if account_info.unsigned_key() == key {
-                    accounts.push(Rc::new(RefCell::new(ai_to_a(account_info))));
+                    accounts.push((*key, Rc::new(RefCell::new(ai_to_a(account_info)))));
                     continue 'outer;
                 }
             }
@@ -336,14 +333,12 @@ impl solana_sdk::program_stubs::SyscallStubs for SyscallStubs {
         .map_err(|err| ProgramError::try_from(err).unwrap_or_else(|err| panic!("{}", err)))?;
 
         // Copy writeable account modifications back into the caller's AccountInfos
-        for (i, account_pubkey) in message.account_keys.iter().enumerate() {
-            if !message.is_writable(i, true) {
+        for (i, (pubkey, account)) in accounts.iter().enumerate().take(message.account_keys.len()) {
+            if !message.is_writable(i) {
                 continue;
             }
-
             for account_info in account_infos {
-                if account_info.unsigned_key() == account_pubkey {
-                    let account = &accounts[i];
+                if account_info.unsigned_key() == pubkey {
                     **account_info.try_borrow_mut_lamports().unwrap() = account.borrow().lamports();
 
                     let mut data = account_info.try_borrow_mut_data()?;
